@@ -1,7 +1,7 @@
 /**************************************************************************
  *   cut.c  --  This file is part of GNU nano.                            *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2021 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2022 Free Software Foundation, Inc.    *
  *   Copyright (C) 2014 Mark Majeres                                      *
  *   Copyright (C) 2016, 2018-2020 Benno Schulenberg                      *
  *                                                                        *
@@ -97,6 +97,13 @@ void do_deletion(undo_type action)
 		/* We're at the end-of-file: nothing to do. */
 		return;
 
+#ifdef ENABLE_COLOR
+	if (!refresh_needed)
+		check_the_multis(openfile->current);
+#endif
+	if (!refresh_needed)
+		update_line(openfile->current, openfile->current_x);
+
 	/* Adjust the file size, and remember it for a possible redo. */
 	openfile->totsize--;
 #ifndef NANO_TINY
@@ -435,9 +442,19 @@ void ingraft_buffer(linestruct *topline)
 /* Meld a copy of the given buffer into the current file buffer. */
 void copy_from_buffer(linestruct *somebuffer)
 {
+#ifdef ENABLE_COLOR
+	size_t threshold = openfile->edittop->lineno + editwinrows - 1;
+#endif
 	linestruct *the_copy = copy_buffer(somebuffer);
 
 	ingraft_buffer(the_copy);
+
+#ifdef ENABLE_COLOR
+	if (openfile->current->lineno > threshold || ISSET(SOFTWRAP))
+		recook = TRUE;
+	else
+		perturbed = TRUE;
+#endif
 }
 
 #ifndef NANO_TINY
@@ -485,7 +502,7 @@ void do_snip(bool marked, bool until_eof, bool append)
 		/* When not at the end of a line, move the rest of this line into
 		 * the cutbuffer.  Otherwise, when not at the end of the buffer,
 		 * move just the "line separator" into the cutbuffer. */
-		if (openfile->current_x < strlen(openfile->current->data))
+		if (line->data[openfile->current_x] != '\0')
 			extract_segment(line, openfile->current_x, line, strlen(line->data));
 		else if (openfile->current != openfile->filebot) {
 			extract_segment(line, openfile->current_x, line->next, 0);
@@ -509,6 +526,9 @@ void do_snip(bool marked, bool until_eof, bool append)
 
 	set_modified();
 	refresh_needed = TRUE;
+#ifdef ENABLE_COLOR
+	perturbed = TRUE;
+#endif
 }
 
 /* Move text from the current buffer into the cutbuffer. */
