@@ -1,7 +1,7 @@
 /**************************************************************************
  *   utils.c  --  This file is part of GNU nano.                          *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2021 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2022 Free Software Foundation, Inc.    *
  *   Copyright (C) 2016, 2017, 2019 Benno Schulenberg                     *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
@@ -130,9 +130,9 @@ bool parse_num(const char *string, ssize_t *result)
  * *line and *column.  Return FALSE on error, and TRUE otherwise. */
 bool parse_line_column(const char *str, ssize_t *line, ssize_t *column)
 {
-	bool retval;
-	char *firstpart;
 	const char *comma;
+	char *firstpart;
+	bool retval;
 
 	while (*str == ' ')
 		str++;
@@ -168,14 +168,19 @@ void recode_NUL_to_LF(char *string, size_t length)
 	}
 }
 
-/* In the given string, recode each embedded newline as a NUL. */
-void recode_LF_to_NUL(char *string)
+/* In the given string, recode each embedded newline as a NUL,
+ * and return the number of bytes in the string. */
+size_t recode_LF_to_NUL(char *string)
 {
+	char *beginning = string;
+
 	while (*string != '\0') {
 		if (*string == '\n')
 			*string = '\0';
 		string++;
 	}
+
+	return (string - beginning);
 }
 
 #if !defined(ENABLE_TINY) || defined(ENABLE_TABCOMP) || defined(ENABLE_BROWSER)
@@ -282,34 +287,26 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 		return mbstrcasestr(start, needle);
 }
 
-/* This is a wrapper for the malloc() function that properly handles
- * things when we run out of memory. */
+/* Allocate the given amount of memory and return a pointer to it. */
 void *nmalloc(size_t howmuch)
 {
-	void *r = malloc(howmuch);
+	void *section = malloc(howmuch);
 
-	if (howmuch == 0)
-		die("Allocating zero bytes.  Please report a bug.\n");
-
-	if (r == NULL)
+	if (section == NULL)
 		die(_("Nano is out of memory!\n"));
 
-	return r;
+	return section;
 }
 
-/* This is a wrapper for the realloc() function that properly handles
- * things when we run out of memory. */
-void *nrealloc(void *ptr, size_t howmuch)
+/* Reallocate the given section of memory to have the given size. */
+void *nrealloc(void *section, size_t howmuch)
 {
-	void *r = realloc(ptr, howmuch);
+	section = realloc(section, howmuch);
 
-	if (howmuch == 0)
-		die("Allocating zero bytes.  Please report a bug.\n");
-
-	if (r == NULL)
+	if (section == NULL)
 		die(_("Nano is out of memory!\n"));
 
-	return r;
+	return section;
 }
 
 /* Return an appropriately reallocated dest string holding a copy of src.
@@ -492,7 +489,9 @@ void get_range(linestruct **top, linestruct **bot)
 			also_the_last = TRUE;
 	}
 }
+#endif /* !NANO_TINY */
 
+#if !defined(NANO_TINY) || defined(ENABLE_SPELLER) || defined (ENABLE_LINTER) || defined (ENABLE_FORMATTER)
 /* Return a pointer to the line that has the given line number. */
 linestruct *line_from_number(ssize_t number)
 {
@@ -507,7 +506,7 @@ linestruct *line_from_number(ssize_t number)
 
 	return line;
 }
-#endif /* !NANO_TINY */
+#endif
 
 /* Count the number of characters from begin to end, and return it. */
 size_t number_of_characters_in(const linestruct *begin, const linestruct *end)
